﻿using System;
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Helpers;

namespace Nikolz
{
    [HandlerCategory("Nikolz")]
    public class SessionHeld : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            var bars = source.Bars;
            var res = new double[bars.Count];
            var curHeld = 0;
            for(int i = 1; i < res.Length; i++)
            {
                if(bars[i-1].Date.Day != bars[i].Date.Day)
                {
                    curHeld = 0;
                }
                else
                {
                    curHeld++;
                }
                res[i] = curHeld;
            }
            return res;
        }
    }

    [HandlerCategory("Nikolz")]
    public class SessionOpen : IBar2DoubleHandler
    {
        [HandlerParameter(true, "1", Min = "0", Max = "10", Step = "1")]
        public int Session
        {
            get;
            set;
        }

        public IList<double> Execute(ISecurity source)
        {
            var bars = source.Bars;
            var res = new double[bars.Count];
            if (res.Length > 0)
            {
                var curRes = new List<double>();
                for (int i = 0; i <= Session; i++)
                {
                    curRes.Add(bars[0].Open);
                }
                for (int i = 1; i < res.Length; i++)
                {
                    if (bars[i - 1].Date.Day != bars[i].Date.Day)
                    {
                        curRes.RemoveAt(0);
                        curRes.Add(bars[i].Open);
                    }
                    res[i] = curRes[0];
                }
            }
            return res;
        }
    }

    [HandlerCategory("Nikolz")]
    public class SessionClose : IBar2DoubleHandler
    {
        [HandlerParameter(true, "1", Min = "0", Max = "10", Step = "1")]
        public int Session
        {
            get;
            set;
        }

        public IList<double> Execute(ISecurity source)
        {
            var bars = source.Bars;
            var res = new double[bars.Count];
            if (res.Length > 0)
            {
                var curRes = new List<double>();
                for (int i = 0; i <= Session; i++)
                {
                    curRes.Add(bars[0].Open);
                }
                for (int i = 1; i < res.Length; i++)
                {
                    if (bars[i - 1].Date.Day != bars[i].Date.Day)
                    {
                        curRes.RemoveAt(0);
                        curRes.Add(bars[i].Close);
                    }
                    curRes[Session] = bars[i].Close;
                    res[i] = curRes[0];
                }
            }
            return res;
        }
    }

    [HandlerCategory("Nikolz")]
    public class SessionHigh : IBar2DoubleHandler
    {
        [HandlerParameter(true, "1", Min = "0", Max = "10", Step = "1")]
        public int Session
        {
            get;
            set;
        }

        public IList<double> Execute(ISecurity source)
        {
            var bars = source.Bars;
            var res = new double[bars.Count];
            if (res.Length > 0)
            {
                var curRes = new List<double>();
                for (int i = 0; i <= Session; i++)
                {
                    curRes.Add(bars[0].High);
                }
                for (int i = 1; i < res.Length; i++)
                {
                    if (bars[i - 1].Date.Day != bars[i].Date.Day)
                    {
                        curRes.RemoveAt(0);
                        curRes.Add(bars[i].High);
                    }
                    curRes[Session] = Math.Max(curRes[Session], bars[i].High);
                    res[i] = curRes[0];
                }
            }
            return res;
        }
    }

    [HandlerCategory("Nikolz")]
    public class SessionLow : IBar2DoubleHandler
    {
        [HandlerParameter(true, "1", Min = "0", Max = "10", Step = "1")]
        public int Session
        {
            get;
            set;
        }

        public IList<double> Execute(ISecurity source)
        {
            var bars = source.Bars;
            var res = new double[bars.Count];
            if (res.Length > 0)
            {
                var curRes = new List<double>();
                for (int i = 0; i <= Session; i++)
                {
                    curRes.Add(bars[0].Low);
                }
                for (int i = 1; i < res.Length; i++)
                {
                    if (bars[i - 1].Date.Day != bars[i].Date.Day)
                    {
                        curRes.RemoveAt(0);
                        curRes.Add(bars[i].Low);
                    }
                    curRes[Session] = Math.Min(curRes[Session], bars[i].Low);
                    res[i] = curRes[0];
                }
            }
            return res;
        }
    }

    [HandlerCategory("Nikolz")]
    public class HighestPos : IDouble2DoubleHandler
    {
        [HandlerParameter(true, "20", Min = "10", Max = "100", Step = "5")]
        public int Period
        {
            get;
            set;
        }

        public IList<double> Execute(IList<double> source)
        {
            var highest = Series.Highest(source, Period);
            var res = new double[source.Count];
            for(int i = 1 ; i < res.Length; i++)
            {
                var k = 0;
                while(i-k >= 0 && highest[i] != source[i-k])
                {
                    k++;
                }
                res[i] = k;
            }
            return res;
        }
    }

    [HandlerCategory("Nikolz")]
    public class LowestPos : IDouble2DoubleHandler
    {
        [HandlerParameter(true, "20", Min = "10", Max = "100", Step = "5")]
        public int Period
        {
            get;
            set;
        }

        public IList<double> Execute(IList<double> source)
        {
            var lowest = Series.Lowest(source, Period);
            var res = new double[source.Count];
            for (int i = 1; i < res.Length; i++)
            {
                var k = 0;
                while (i - k >= 0 && lowest[i] != source[i - k])
                {
                    k++;
                }
                res[i] = k;
            }
            return res;
        }
    }
}
